@@ -19,8 +19,10 @@
 
 #include "CBot/CBotClass.h"
 
+#include "CBot/CBotInstr/CBotInstrUtils.h"
 #include "CBot/CBotInstr/CBotNew.h"
 #include "CBot/CBotInstr/CBotLeftExprVar.h"
+#include "CBot/CBotInstr/CBotExprLitNull.h"
 #include "CBot/CBotInstr/CBotTwoOpExpr.h"
 #include "CBot/CBotInstr/CBotFunction.h"
 #include "CBot/CBotInstr/CBotExpression.h"
@@ -32,6 +34,7 @@
 #include "CBot/CBotExternalCall.h"
 #include "CBot/CBotStack.h"
 #include "CBot/CBotCStack.h"
+#include "CBot/CBotDefParam.h"
 #include "CBot/CBotUtils.h"
 #include "CBot/CBotFileUtils.h"
 #include "CBot/CBotCallMethode.h"
@@ -89,7 +92,7 @@ void CBotClass::ClearPublic()
 ////////////////////////////////////////////////////////////////////////////////
 void CBotClass::Purge()
 {
-    if ( this == nullptr ) return;
+    assert ( this != nullptr );
 
     delete      m_pVar;
     m_pVar      = nullptr;
@@ -101,7 +104,7 @@ void CBotClass::Purge()
 
     m_nbVar     = m_parent == nullptr ? 0 : m_parent->m_nbVar;
 
-    m_next->Purge();
+    if (m_next != nullptr) m_next->Purge();
     m_next = nullptr;          // no longer belongs to this chain
 }
 
@@ -202,7 +205,7 @@ std::string  CBotClass::GetName()
 ////////////////////////////////////////////////////////////////////////////////
 CBotClass*  CBotClass::GetParent()
 {
-    if ( this == nullptr ) return nullptr;
+    assert ( this != nullptr );
     return m_parent;
 }
 
@@ -330,7 +333,7 @@ CBotTypResult CBotClass::CompileMethode(const std::string& name,
 
     r = m_pMethod->CompileCall(name, ppParams, nIdent);
     if ( r.Eq(CBotErrUndefCall) && m_parent != nullptr )
-        return m_parent->m_pMethod->CompileCall(name, ppParams, nIdent);
+        return m_parent->CompileMethode(name, pThis, ppParams, pStack, nIdent);
     return r;
 }
 
@@ -351,9 +354,7 @@ bool CBotClass::ExecuteMethode(long& nIdent,
 
     if (m_parent != nullptr)
     {
-        ret = m_parent->m_pCalls->DoCall(name, pThis, ppParams, pResult, pStack, pToken);
-        if (ret >= 0) return ret;
-        ret = m_parent->m_pMethod->DoCall(nIdent, name, pThis, ppParams, pStack, pToken, m_parent);
+        ret = m_parent->ExecuteMethode(nIdent, name, pThis, ppParams, pResult, pStack, pToken);
     }
     return ret;
 }
@@ -365,7 +366,14 @@ void CBotClass::RestoreMethode(long& nIdent,
                                CBotVar** ppParams,
                                CBotStack*& pStack)
 {
-    m_pMethod->RestoreCall(nIdent, name, pThis, ppParams, pStack, this);
+    CBotClass* pClass = this;
+    while (pClass != nullptr)
+    {
+        bool ok = pClass->m_pMethod->RestoreCall(nIdent, name, pThis, ppParams, pStack, pClass);
+        if (ok) return;
+        pClass = pClass->m_parent;
+    }
+    assert(false);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -501,16 +509,25 @@ CBotClass* CBotClass::Compile1(CBotToken* &p, CBotCStack* pStack)
         classe->Purge();                            // empty the old definitions // TODO: Doesn't this remove all classes of the current program?
         classe->m_IsDef = false;                    // current definition
 
+        classe->m_pOpenblk = p;
+
         if ( !IsOfType( p, ID_OPBLK) )
         {
             pStack->SetError(CBotErrOpenBlock, p);
             return nullptr;
         }
 
-        while ( pStack->IsOk() && !IsOfType( p, ID_CLBLK ) )
+        int level = 1;
+        do                                          // skip over the definition
         {
-            classe->CompileDefItem(p, pStack, false);
+            int type = p->GetType();
+            p = p->GetNext();
+            if (type == ID_OPBLK) level++;
+            if (type == ID_CLBLK) level--;
         }
+        while (level > 0 && p != nullptr);
+
+        if (level > 0) pStack->SetError(CBotErrCloseBlock, classe->m_pOpenblk);
 
         if (pStack->IsOk()) return classe;
     }
@@ -518,6 +535,26 @@ CBotClass* CBotClass::Compile1(CBotToken* &p, CBotCStack* pStack)
     return nullptr;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+void CBotClass::DefineClasses(CBotClass* pClass, CBotCStack* pStack)
+{
+    while (pClass != nullptr)
+    {
+        CBotClass* pParent = pClass->m_parent;
+        pClass->m_nbVar = (pParent == nullptr) ? 0 : pParent->m_nbVar;
+        CBotToken* p = pClass->m_pOpenblk->GetNext();
+
+        while (pStack->IsOk() && !IsOfType(p, ID_CLBLK))
+        {
+            pClass->CompileDefItem(p, pStack, false);
+        }
+
+        if (!pStack->IsOk()) return;
+
+        pClass = pClass->GetNext();
+    }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
 {
@@ -549,6 +586,7 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
 
     while (pStack->IsOk())
     {
+        CBotTypResult  type2 = CBotTypResult(type);                     // reset type after comma
         std::string pp = p->GetString();
         if ( IsOfType(p, ID_NOT) )
         {
@@ -561,29 +599,27 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
             while ( IsOfType( p, ID_OPBRK ) )   // a table?
             {
                 CBotInstr* i = nullptr;
-
+                pStack->SetStartError( p->GetStart() );
                 if ( p->GetType() != ID_CLBRK )
+                {
                     i = CBotExpression::Compile( p, pStack );           // expression for the value
+                    if (i == nullptr || pStack->GetType() != CBotTypInt) // must be a number
+                    {
+                        pStack->SetError(CBotErrBadIndex, p->GetStart());
+                        return false;
+                    }
+                }
                 else
                     i = new CBotEmpty();                            // special if not a formula
 
-                type = CBotTypResult(CBotTypArrayPointer, type);
-
-                if (!pStack->IsOk() || !IsOfType( p, ID_CLBRK ) )
-                {
-                    pStack->SetError(CBotErrCloseIndex, p->GetStart());
-                    return false;
-                }
-
-/*              CBotVar* pv = pStack->GetVar();
-                if ( pv->GetType()>= CBotTypBoolean )
-                {
-                    pStack->SetError(CBotErrBadType1, p->GetStart());
-                    return false;
-                }*/
+                type2 = CBotTypResult(CBotTypArrayPointer, type2);
 
                 if (limites == nullptr) limites = i;
                 else limites->AddNext3(i);
+
+                if (IsOfType(p, ID_CLBRK)) continue;
+                pStack->SetError(CBotErrCloseIndex, p->GetStart());
+                return false;
             }
 
             if ( p->GetType() == ID_OPENPAR )
@@ -603,11 +639,14 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
                 {
                     // return a method precompiled in pass 1
                     CBotFunction*   pf = m_pMethod;
-                    CBotFunction*   prev = nullptr;
-                    while ( pf != nullptr )
+                    CBotToken* ppp = p;
+                    CBotCStack* pStk = pStack->TokenStack(nullptr, true);
+                    CBotDefParam* params = CBotDefParam::Compile(p, pStk );
+                    delete pStk;
+                    p = ppp;
+                    while ( pf != nullptr )                             // search by name and parameters
                     {
-                        if (pf->GetName() == pp) break;
-                        prev = pf;
+                        if (pf->GetName() == pp && pf->CheckParam( params )) break;
                         pf = pf->Next();
                     }
 
@@ -643,6 +682,7 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
                                 initType = CBotVar::InitType::DEF;
                             pcopy->SetInit(initType);
                             pcopy->SetUniqNum(pv->GetUniqNum());
+                            pcopy->SetPrivate(pv->GetPrivate());
                             pile->AddVar(pcopy);
                             pv = pv->GetNext();
                         }
@@ -652,18 +692,12 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
                     // compiles a method
                     p = pBase;
                     CBotFunction* f =
-                    CBotFunction::Compile(p, pile, nullptr/*, false*/);
+                    CBotFunction::Compile(p, pile, pf/*, false*/);
 
                     if ( f != nullptr )
                     {
                         f->m_pProg = pStack->GetProgram();
                         f->m_bSynchro = bSynchro;
-                        // replaces the element in the chain
-                        f->m_next = pf->m_next;
-                        pf->m_next = nullptr;
-                        delete pf;
-                        if (prev == nullptr) m_pMethod = f;
-                        else prev->m_next = f;
                     }
                     pStack->Return(nullptr, pile);
                 }
@@ -672,7 +706,7 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
             }
 
             // definition of an element
-            if (type.Eq(0))
+            if (type2.Eq(0))
             {
                 pStack->SetError(CBotErrNoTerminator, p);
                 return false;
@@ -681,22 +715,47 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
             CBotInstr* i = nullptr;
             if ( IsOfType(p, ID_ASS ) )
             {
-                if ( type.Eq(CBotTypArrayPointer) )
+                pStack->SetStartError(p->GetStart());
+                if ( IsOfType(p, ID_SEP) )
                 {
-                    i = CBotListArray::Compile(p, pStack, type.GetTypElem());
+                    pStack->SetError(CBotErrNoExpression, p->GetStart());
+                    return false;
+                }
+                if ( type2.Eq(CBotTypArrayPointer) )
+                {
+                    if ( nullptr == (i = CBotListArray::Compile(p, pStack, type2.GetTypElem())) )
+                    {
+                        if (pStack->IsOk())
+                        {
+                            i = CBotTwoOpExpr::Compile(p, pStack);
+                            if (i == nullptr || !pStack->GetTypResult().Compare(type2))
+                            {
+                                pStack->SetError(CBotErrBadType1, p->GetStart());
+                                return false;
+                            }
+                        }
+                    }
                 }
                 else
                 {
                     // it has an assignmet to calculate
                     i = CBotTwoOpExpr::Compile(p, pStack);
+
+                    if ( !(type.Eq(CBotTypPointer) && pStack->GetTypResult().Eq(CBotTypNullPointer)) &&
+                         !TypesCompatibles( type2, pStack->GetTypResult()) )
+                    {
+                        pStack->SetError(CBotErrBadType1, p->GetStart());
+                        return false;
+                    }
                 }
                 if ( !pStack->IsOk() ) return false;
             }
+            else if ( type2.Eq(CBotTypArrayPointer) ) i = new CBotExprLitNull();
 
 
             if ( !bSecond )
             {
-                CBotVar*    pv = CBotVar::Create(pp, type);
+                CBotVar*    pv = CBotVar::Create(pp, type2);
                 pv -> SetStatic( bStatic );
                 pv -> SetPrivate( mProtect );
 
@@ -709,8 +768,15 @@ bool CBotClass::CompileDefItem(CBotToken* &p, CBotCStack* pStack, bool bSecond)
                 if ( pv->IsStatic() && pv->m_InitExpr != nullptr )
                 {
                     CBotStack* pile = CBotStack::AllocateStack();              // independent stack
-                    while(pile->IsOk() && !pv->m_InitExpr->Execute(pile));  // evaluates the expression without timer
-                    pv->SetVal( pile->GetVar() ) ;
+                    if ( type2.Eq(CBotTypArrayPointer) )
+                    {
+                        while(pile->IsOk() && !pv->m_InitExpr->Execute(pile, pv));
+                    }
+                    else
+                    {
+                        while(pile->IsOk() && !pv->m_InitExpr->Execute(pile)); // evaluates the expression without timer
+                        pv->SetVal( pile->GetVar() ) ;
+                    }
                     pile->Delete();
                 }
             }
