@@ -224,10 +224,6 @@ CBotFunction* CBotFunction::Compile(CBotToken* &p, CBotCStack* pStack, CBotFunct
                 func->m_closeblk = (p != nullptr && p->GetPrev() != nullptr) ? *(p->GetPrev()) : CBotToken();
                 if ( pStk->IsOk() )
                 {
-                    if ( func->m_bPublic )  // public function, return known for all
-                    {
-                        CBotFunction::AddPublic(func);
-                    }
                     return pStack->ReturnFunc(func, pStk);
                 }
             }
@@ -466,8 +462,7 @@ CBotFunction* CBotFunction::FindLocalOrPublic(long& nIdent, const std::string& n
 
     if ( name.empty() ) return nullptr;
 
-    int     delta   = 99999;                // seeks the lowest signature
-    CBotFunction*   pFunc = nullptr;           // the best function found
+    std::map<CBotFunction*, int> funcMap;
 
     if ( this != nullptr )
     {
@@ -482,44 +477,48 @@ CBotFunction* CBotFunction::FindLocalOrPublic(long& nIdent, const std::string& n
                 CBotVar* pw = ppVars[i++];              // provided list parameter
                 while ( pv != nullptr && pw != nullptr)
                 {
-                    if (!TypesCompatibles(pv->GetTypResult(), pw->GetTypResult()))
+                    CBotTypResult paramType = pv->GetTypResult();
+                    CBotTypResult argType = pw->GetTypResult(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
+
+                    if (!TypesCompatibles(paramType, argType))
                     {
-                        if ( pFunc == nullptr ) TypeOrError = CBotErrBadParam;
+                        if ( funcMap.empty() ) TypeOrError.SetType(CBotErrBadParam);
                         break;
                     }
-                    int d = pv->GetType() - pw->GetType(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
-                    alpha += d>0 ? d : -10*d;       // quality loss, 10 times more expensive!
 
+                    if (paramType.Eq(CBotTypPointer) && !argType.Eq(CBotTypNullPointer))
+                    {
+                        CBotClass* c1 = paramType.GetClass();
+                        CBotClass* c2 = argType.GetClass();
+                        while (c2 != c1 && c2 != nullptr)    // implicit cast
+                        {
+                            alpha += 10;
+                            c2 = c2->GetParent();
+                        }
+                    }
+                    else
+                    {
+                        int d = pv->GetType() - pw->GetType(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
+                        alpha += d>0 ? d : -10*d;       // quality loss, 10 times more expensive!
+                    }
                     pv = pv->GetNext();
                     pw = ppVars[i++];
                 }
                 if ( pw != nullptr )
                 {
-                    if ( pFunc != nullptr ) continue;
+                    if ( !funcMap.empty() ) continue;
                     if ( TypeOrError.Eq(CBotErrLowParam) ) TypeOrError.SetType(CBotErrNbParam);
                     if ( TypeOrError.Eq(CBotErrUndefCall)) TypeOrError.SetType(CBotErrOverParam);
                     continue;                   // too many parameters
                 }
                 if ( pv != nullptr )
                 {
-                    if ( pFunc != nullptr ) continue;
+                    if ( !funcMap.empty() ) continue;
                     if ( TypeOrError.Eq(CBotErrOverParam) ) TypeOrError.SetType(CBotErrNbParam);
                     if ( TypeOrError.Eq(CBotErrUndefCall) ) TypeOrError.SetType(CBotErrLowParam);
                     continue;                   // not enough parameters
                 }
-
-                if (alpha == 0)                 // perfect signature
-                {
-                    nIdent = pt->m_nFuncIdent;
-                    TypeOrError = pt->m_retTyp;
-                    return pt;
-                }
-
-                if ( alpha < delta )            // a better signature?
-                {
-                    pFunc = pt;
-                    delta = alpha;
-                }
+                funcMap.insert( std::pair<CBotFunction*, int>(pt, alpha) );
             }
         }
     }
@@ -537,50 +536,72 @@ CBotFunction* CBotFunction::FindLocalOrPublic(long& nIdent, const std::string& n
                 CBotVar* pw = ppVars[i++];              // list of provided parameters
                 while ( pv != nullptr && pw != nullptr)
                 {
-                    if (!TypesCompatibles(pv->GetTypResult(), pw->GetTypResult()))
+                    CBotTypResult paramType = pv->GetTypResult();
+                    CBotTypResult argType = pw->GetTypResult(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
+
+                    if (!TypesCompatibles(paramType, argType))
                     {
-                        if ( pFunc == nullptr ) TypeOrError = CBotErrBadParam;
+                        if ( funcMap.empty() ) TypeOrError.SetType(CBotErrBadParam);
                         break;
                     }
-                    int d = pv->GetType() - pw->GetType(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
-                    alpha += d>0 ? d : -10*d;       // quality loss, 10 times more expensive!
 
+                    if (paramType.Eq(CBotTypPointer) && !argType.Eq(CBotTypNullPointer))
+                    {
+                        CBotClass* c1 = paramType.GetClass();
+                        CBotClass* c2 = argType.GetClass();
+                        while (c2 != c1 && c2 != nullptr)    // implicit cast
+                        {
+                            alpha += 10;
+                            c2 = c2->GetParent();
+                        }
+                    }
+                    else
+                    {
+                        int d = pv->GetType() - pw->GetType(CBotVar::GetTypeMode::CLASS_AS_INTRINSIC);
+                        alpha += d>0 ? d : -10*d;       // quality loss, 10 times more expensive!
+                    }
                     pv = pv->GetNext();
                     pw = ppVars[i++];
                 }
                 if ( pw != nullptr )
                 {
-                    if ( pFunc != nullptr ) continue;
+                    if ( !funcMap.empty() ) continue; // previous useable function
                     if ( TypeOrError.Eq(CBotErrLowParam) ) TypeOrError.SetType(CBotErrNbParam);
                     if ( TypeOrError.Eq(CBotErrUndefCall)) TypeOrError.SetType(CBotErrOverParam);
                     continue;                   // to many parameters
                 }
                 if ( pv != nullptr )
                 {
-                    if ( pFunc != nullptr ) continue;
+                    if ( !funcMap.empty() ) continue; // previous useable function
                     if ( TypeOrError.Eq(CBotErrOverParam) ) TypeOrError.SetType(CBotErrNbParam);
                     if ( TypeOrError.Eq(CBotErrUndefCall) ) TypeOrError.SetType(CBotErrLowParam);
                     continue;                   // not enough parameters
                 }
-
-                if (alpha == 0)                 // perfect signature
-                {
-                    nIdent = pt->m_nFuncIdent;
-                    TypeOrError = pt->m_retTyp;
-                    return pt;
-                }
-
-                if ( alpha < delta )            // a better signature?
-                {
-                    pFunc = pt;
-                    delta = alpha;
-                }
+                funcMap.insert( std::pair<CBotFunction*, int>(pt, alpha) );
             }
         }
     }
 
-    if ( pFunc != nullptr )
+    if ( !funcMap.empty() )
     {
+        auto it = funcMap.begin();
+        CBotFunction* pFunc = it->first;        // the best function found
+        signed int    delta = it->second;       // seeks the lowest signature
+
+        for (++it ; it != funcMap.end() ; it++)
+        {
+            if (it->second < delta) // a better signature?
+            {
+                TypeOrError.SetType(CBotNoErr);
+                pFunc = it->first;
+                delta = it->second;
+                continue;
+            }
+
+            if (it->second == delta) TypeOrError.SetType(CBotErrAmbiguousCall);
+        }
+
+        if (TypeOrError.Eq(CBotErrAmbiguousCall)) return nullptr;
         nIdent = pFunc->m_nFuncIdent;
         TypeOrError = pFunc->m_retTyp;
         return pFunc;
@@ -801,7 +822,7 @@ int CBotFunction::DoCall(long& nIdent, const std::string& name, CBotVar* pThis,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void CBotFunction::RestoreCall(long& nIdent, const std::string& name, CBotVar* pThis, CBotVar** ppVars,
+bool CBotFunction::RestoreCall(long& nIdent, const std::string& name, CBotVar* pThis, CBotVar** ppVars,
                                CBotStack* pStack, CBotClass* pClass)
 {
     CBotTypResult   type;
@@ -810,14 +831,20 @@ void CBotFunction::RestoreCall(long& nIdent, const std::string& name, CBotVar* p
     if ( pt != nullptr )
     {
         CBotStack*  pStk = pStack->RestoreStack(pt);
-        if ( pStk == nullptr ) return;
+        if ( pStk == nullptr ) return true;
         pStk->SetProgram(pt->m_pProg);                  // it may have changed module
 
         CBotVar*    pthis = pStk->FindVar("this");
         pthis->SetUniqNum(-2);
 
+        if (pClass->GetParent() != nullptr)
+        {
+            CBotVar* psuper = pStk->FindVar("super");
+            if (psuper != nullptr) psuper->SetUniqNum(-3);
+        }
+
         CBotStack*  pStk3 = pStk->RestoreStack(nullptr);   // to set parameters passed
-        if ( pStk3 == nullptr ) return;
+        if ( pStk3 == nullptr ) return true;
 
         pt->m_param->RestoreState(pStk3, true);                 // parameters
 
@@ -831,7 +858,9 @@ void CBotFunction::RestoreCall(long& nIdent, const std::string& name, CBotVar* p
         // finally calls the found function
 
         pt->m_block->RestoreState(pStk3, true);                 // interrupt !
+        return true;
     }
+    return false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -840,8 +869,8 @@ bool CBotFunction::CheckParam(CBotDefParam* pParam)
     CBotDefParam*   pp = m_param;
     while ( pp != nullptr && pParam != nullptr )
     {
-        CBotTypResult type1 = pp->GetType();
-        CBotTypResult type2 = pParam->GetType();
+        CBotTypResult type1 = pp->GetTypResult();
+        CBotTypResult type2 = pParam->GetTypResult();
         if ( !type1.Compare(type2) ) return false;
         pp = pp->GetNext();
         pParam = pParam->GetNext();
