@@ -21,19 +21,23 @@
 
 #include "app/controller.h"
 #include "app/input.h"
-#include "app/system.h"
+#include "app/pathman.h"
 
 #include "common/config_file.h"
 #include "common/image.h"
 #include "common/key.h"
 #include "common/logger.h"
 #include "common/make_unique.h"
-#include "common/pathman.h"
+#include "common/profiler.h"
 #include "common/stringutils.h"
 #include "common/version.h"
 
 #include "common/resources/resourcemanager.h"
 
+#include "common/system/system.h"
+
+#include "common/thread/thread.h"
+
 #include "graphics/core/nulldevice.h"
 
 #include "graphics/opengl/glutil.h"
@@ -42,11 +46,11 @@
 
 #include "object/object_manager.h"
 
+#include "sound/sound.h"
 #ifdef OPENAL_SOUND
     #include "sound/oalsound/alsound.h"
 #endif
 
-#include <boost/filesystem.hpp>
 #include <boost/tokenizer.hpp>
 
 #include <SDL.h>
@@ -112,9 +116,7 @@ CApplication::CApplication(CSystemUtils* systemUtils)
       m_private(MakeUnique<ApplicationPrivate>()),
       m_configFile(MakeUnique<CConfigFile>()),
       m_input(MakeUnique<CInput>()),
-      m_pathManager(MakeUnique<CPathManager>(systemUtils)),
-      m_performanceCounters(),
-      m_performanceCountersData()
+      m_pathManager(MakeUnique<CPathManager>(systemUtils))
 {
     m_exitCode      = 0;
     m_active        = false;
@@ -144,11 +146,6 @@ CApplication::CApplication(CSystemUtils* systemUtils)
     m_manualFrameLast = m_systemUtils->CreateTimeStamp();
     m_manualFrameTime = m_systemUtils->CreateTimeStamp();
 
-    for (int i = 0; i < PCNT_MAX; ++i)
-    {
-        m_performanceCounters[i][0] = m_systemUtils->CreateTimeStamp();
-        m_performanceCounters[i][1] = m_systemUtils->CreateTimeStamp();
-    }
 
     m_joystickEnabled = false;
 
@@ -173,12 +170,6 @@ CApplication::~CApplication()
     m_systemUtils->DestroyTimeStamp(m_manualFrameLast);
     m_systemUtils->DestroyTimeStamp(m_manualFrameTime);
 
-    for (int i = 0; i < PCNT_MAX; ++i)
-    {
-        m_systemUtils->DestroyTimeStamp(m_performanceCounters[i][0]);
-        m_systemUtils->DestroyTimeStamp(m_performanceCounters[i][1]);
-    }
-
     m_joystickEnabled = false;
 
     m_controller.reset();
@@ -514,8 +505,6 @@ bool CApplication::Create()
     #endif
 
     m_sound->Create();
-    m_sound->CacheAll();
-    m_sound->CacheCommonMusic();
 
     GetLogger()->Info("CApplication created successfully\n");
 
@@ -685,6 +674,20 @@ bool CApplication::Create()
     // Create the robot application.
     m_controller = MakeUnique<CController>();
 
+    CThread musicLoadThread([this]() {
+        GetLogger()->Debug("Cache sounds...\n");
+        SystemTimeStamp* musicLoadStart = m_systemUtils->CreateTimeStamp();
+        m_systemUtils->GetCurrentTimeStamp(musicLoadStart);
+
+        m_sound->CacheAll();
+
+        SystemTimeStamp* musicLoadEnd = m_systemUtils->CreateTimeStamp();
+        m_systemUtils->GetCurrentTimeStamp(musicLoadEnd);
+        float musicLoadTime = m_systemUtils->TimeStampDiff(musicLoadStart, musicLoadEnd, STU_MSEC);
+        GetLogger()->Debug("Sound loading took %.2f ms\n", musicLoadTime);
+    }, "Sound loading thread");
+    musicLoadThread.Start();
+
     if (m_runSceneCategory == LevelCategory::Max)
         m_controller->StartApp();
     else
@@ -833,7 +836,7 @@ bool CApplication::ChangeVideoConfig(const Gfx::DeviceConfig &newConfig)
 
     m_device->ConfigChanged(m_deviceConfig);
 
-    m_engine->ResetAfterVideoConfigChanged();
+    m_eventQueue->AddEvent(Event(EVENT_RESOLUTION_CHANGED));
 
     return true;
 }
@@ -994,12 +997,10 @@ int CApplication::Run()
 
     while (true)
     {
-        ResetPerformanceCounters();
-
         if (m_active)
         {
-            StartPerformanceCounter(PCNT_ALL);
-            StartPerformanceCounter(PCNT_EVENT_PROCESSING);
+            CProfiler::StartPerformanceCounter(PCNT_ALL);
+            CProfiler::StartPerformanceCounter(PCNT_EVENT_PROCESSING);
         }
 
         // To be sure no old event remains
@@ -1088,9 +1089,9 @@ int CApplication::Run()
                     m_controller->ProcessEvent(event);
             }
 
-            StopPerformanceCounter(PCNT_EVENT_PROCESSING);
+            CProfiler::StopPerformanceCounter(PCNT_EVENT_PROCESSING);
 
-            StartPerformanceCounter(PCNT_UPDATE_ALL);
+            CProfiler::StartPerformanceCounter(PCNT_UPDATE_ALL);
 
             // Prepare and process step simulation event
             Event event = CreateUpdateEvent();
@@ -1100,16 +1101,16 @@ int CApplication::Run()
 
                 m_sound->FrameMove(m_relTime);
 
-                StartPerformanceCounter(PCNT_UPDATE_GAME);
+                CProfiler::StartPerformanceCounter(PCNT_UPDATE_GAME);
                 m_controller->ProcessEvent(event);
-                StopPerformanceCounter(PCNT_UPDATE_GAME);
+                CProfiler::StopPerformanceCounter(PCNT_UPDATE_GAME);
 
-                StartPerformanceCounter(PCNT_UPDATE_ENGINE);
+                CProfiler::StartPerformanceCounter(PCNT_UPDATE_ENGINE);
                 m_engine->FrameUpdate();
-                StopPerformanceCounter(PCNT_UPDATE_ENGINE);
+                CProfiler::StopPerformanceCounter(PCNT_UPDATE_ENGINE);
             }
 
-            StopPerformanceCounter(PCNT_UPDATE_ALL);
+            CProfiler::StopPerformanceCounter(PCNT_UPDATE_ALL);
 
             /* Update mouse position explicitly right before rendering
              * because mouse events are usually way behind */
@@ -1117,9 +1118,7 @@ int CApplication::Run()
 
             Render();
 
-            StopPerformanceCounter(PCNT_ALL);
-
-            UpdatePerformanceCountersData();
+            CProfiler::StopPerformanceCounter(PCNT_ALL);
         }
     }
 
@@ -1404,14 +1403,14 @@ Event CApplication::CreateVirtualEvent(const Event& sourceEvent)
 /** Renders the frame and swaps buffers as necessary */
 void CApplication::Render()
 {
-    StartPerformanceCounter(PCNT_RENDER_ALL);
+    CProfiler::StartPerformanceCounter(PCNT_RENDER_ALL);
     m_engine->Render();
-    StopPerformanceCounter(PCNT_RENDER_ALL);
+    CProfiler::StopPerformanceCounter(PCNT_RENDER_ALL);
 
-    StartPerformanceCounter(PCNT_SWAP_BUFFERS);
+    CProfiler::StartPerformanceCounter(PCNT_SWAP_BUFFERS);
     if (m_deviceConfig.doubleBuf)
         SDL_GL_SwapWindow(m_private->window);
-    StopPerformanceCounter(PCNT_SWAP_BUFFERS);
+    CProfiler::StopPerformanceCounter(PCNT_SWAP_BUFFERS);
 }
 
 void CApplication::RenderIfNeeded(int updateRate)
@@ -1812,13 +1811,17 @@ void CApplication::SetLanguage(Language language)
 
     char* defaultLocale = setlocale(LC_ALL, ""); // Load system locale
     GetLogger()->Debug("Default system locale: %s\n", defaultLocale);
+    if (!locale.empty()) // Override system locale?
+    {
+        setlocale(LC_ALL, locale.c_str());
+    }
     setlocale(LC_NUMERIC, "C"); // Force numeric locale to "C" (fixes decimal point problems)
-    char* systemLocale = setlocale(LC_ALL, nullptr); // Get current locale configuration
-    GetLogger()->Debug("Setting locale: %s\n", systemLocale);
+    std::string systemLocale = setlocale(LC_ALL, nullptr); // Get current locale configuration
+    GetLogger()->Debug("Setting locale: %s\n", systemLocale.c_str());
     // Update C++ locale
     try
     {
-        std::locale::global(std::locale(systemLocale));
+        std::locale::global(std::locale(systemLocale.c_str()));
     }
     catch (...)
     {
@@ -1844,45 +1847,6 @@ void CApplication::SetLanguage(Language language)
     GetLogger()->Debug("SetLanguage: Test gettext translation: '%s'\n", gettext("Colobot rules!"));
 }
 
-void CApplication::StartPerformanceCounter(PerformanceCounter counter)
-{
-    m_systemUtils->GetCurrentTimeStamp(m_performanceCounters[counter][0]);
-}
-
-void CApplication::StopPerformanceCounter(PerformanceCounter counter)
-{
-    m_systemUtils->GetCurrentTimeStamp(m_performanceCounters[counter][1]);
-}
-
-float CApplication::GetPerformanceCounterData(PerformanceCounter counter) const
-{
-    return m_performanceCountersData[counter];
-}
-
-void CApplication::ResetPerformanceCounters()
-{
-    for (int i = 0; i < PCNT_MAX; ++i)
-    {
-        StartPerformanceCounter(static_cast<PerformanceCounter>(i));
-        StopPerformanceCounter(static_cast<PerformanceCounter>(i));
-    }
-}
-
-void CApplication::UpdatePerformanceCountersData()
-{
-    long long sum = m_systemUtils->TimeStampExactDiff(m_performanceCounters[PCNT_ALL][0],
-                                                      m_performanceCounters[PCNT_ALL][1]);
-
-    for (int i = 0; i < PCNT_MAX; ++i)
-    {
-        long long diff = m_systemUtils->TimeStampExactDiff(m_performanceCounters[i][0],
-                                                           m_performanceCounters[i][1]);
-
-        m_performanceCountersData[static_cast<PerformanceCounter>(i)] =
-            static_cast<float>(diff) / static_cast<float>(sum);
-    }
-}
-
 bool CApplication::GetSceneTestMode()
 {
     return m_sceneTest;
