@@ -150,7 +150,6 @@ COldObject::COldObject(int id)
     m_character.wheelRight = 1.0f;
 
     m_cameraType = Gfx::CAM_TYPE_BACK;
-    m_cameraDist = 50.0f;
     m_bCameraLock = false;
 
     for (int i=0 ; i<OBJECTMAXPART ; i++ )
@@ -1031,7 +1030,6 @@ void COldObject::Read(CLevelParserLine* line)
 
     if (line->GetParam("camera")->IsDefined())
         SetCameraType(line->GetParam("camera")->AsCameraType());
-    SetCameraDist(line->GetParam("cameraDist")->AsFloat(50.0f));
     SetCameraLock(line->GetParam("cameraLock")->AsBool(false));
 
     if (line->GetParam("pyro")->IsDefined())
@@ -2497,16 +2495,6 @@ Gfx::CameraType COldObject::GetCameraType()
     return m_cameraType;
 }
 
-void COldObject::SetCameraDist(float dist)
-{
-    m_cameraDist = dist;
-}
-
-float COldObject::GetCameraDist()
-{
-    return m_cameraDist;
-}
-
 void COldObject::SetCameraLock(bool lock)
 {
     m_bCameraLock = lock;
@@ -2973,6 +2961,7 @@ void COldObject::UpdateSelectParticle()
     // Updates lens.
     for ( i=0 ; i<4 ; i++ )
     {
+        if (m_partiSel[i] == -1) continue;
         pos[i] = Math::Transform(m_objectPart[0].matWorld, pos[i]);
         dim[i].y = dim[i].x;
         m_particle->SetParam(m_partiSel[i], pos[i], dim[i], zoom[i], angle, 1.0f);
