@@ -44,6 +44,7 @@
 #include "ui/controls/interface.h"
 #include "ui/controls/list.h"
 
+#include <libintl.h>
 
 const int CBOT_IPF = 100;       // CBOT: default number of instructions / frame
 
@@ -70,11 +71,8 @@ CScript::CScript(COldObject* object)
     m_bRun = false;
     m_bStepMode = false;
     m_bCompile = false;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
     m_cursor1 = 0;
     m_cursor2 = 0;
-    m_filename[0] = 0;
 }
 
 // Object's destructor.
@@ -107,9 +105,11 @@ void CScript::PutScript(Ui::CEdit* edit, const char* name)
 bool CScript::GetScript(Ui::CEdit* edit)
 {
     int len = edit->GetTextLength();
-    m_script = MakeUniqueArray<char>(len+1);
+    m_script = MakeUniqueArray<char>(len+2);
+
+    std::string tmp = edit->GetText(len+1);
+    strncpy(m_script.get(), tmp.c_str(), len+1);
 
-    edit->GetText(m_script.get(), len+1);
     edit->GetCursor(m_cursor2, m_cursor1);
     m_len = strlen(m_script.get());
 
@@ -159,49 +159,55 @@ bool CScript::CheckToken()
     if ( !m_object->GetCheckToken() )  return true;
 
     m_error = CBot::CBotNoErr;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
-    m_token[0] = 0;
+    m_title.clear();
+    m_mainFunction.clear();
+    m_token.clear();
     m_bCompile = false;
 
-    std::vector<bool> used(m_main->GetObligatoryToken(), false);
+    std::map<std::string, int> used;
+    std::map<std::string, int> cursor1;
+    std::map<std::string, int> cursor2;
 
     auto tokens = CBot::CBotToken::CompileTokens(m_script.get());
     CBot::CBotToken* bt = tokens.get();
     while ( bt != nullptr )
     {
         std::string token = bt->GetString();
-        int cursor1 = bt->GetStart();
-        int cursor2 = bt->GetEnd();
 
-        int i = m_main->IsObligatoryToken(token.c_str());
-        if ( i != -1 )
-        {
-            used[i] = true;  // token used
-        }
+        // Store only the last occurrence of the token
+        cursor1[token] = bt->GetStart();
+        cursor2[token] = bt->GetEnd();
 
-        if ( !m_main->IsProhibitedToken(token.c_str()) )
-        {
-            m_error = static_cast<CBot::CBotError>(ERR_PROHIBITEDTOKEN);
-            m_cursor1 = cursor1;
-            m_cursor2 = cursor2;
-            strcpy(m_title, "<prohibited>");
-            m_mainFunction[0] = 0;
-            return false;
-        }
+        used[token]++;
 
         bt = bt->GetNext();
     }
 
-    // At least once every obligatory instruction?
-    for (unsigned int i = 0; i < used.size(); i++)
+    for (const auto& it : m_main->GetObligatoryTokenList())
     {
-        if (!used[i])  // token not used?
+        Error error = ERR_OK;
+        int allowed = 0;
+        if (it.second.max >= 0 && used[it.first] > it.second.max)
         {
-            strcpy(m_token, m_main->GetObligatoryToken(i));
-            m_error = static_cast<CBot::CBotError>(ERR_OBLIGATORYTOKEN);
-            strcpy(m_title, "<obligatory>");
-            m_mainFunction[0] = 0;
+            error = ERR_PROHIBITEDTOKEN;
+            allowed = it.second.max;
+            m_cursor1 = cursor1[it.first];
+            m_cursor2 = cursor2[it.first];
+        }
+        if (it.second.min >= 0 && used[it.first] < it.second.min)
+        {
+            error = ERR_OBLIGATORYTOKEN;
+            allowed = it.second.min;
+        }
+
+        if (error != ERR_OK)
+        {
+            m_token = it.first;
+            m_tokenUsed = used[it.first];
+            m_tokenAllowed = allowed;
+            m_error = static_cast<CBot::CBotError>(error);
+            m_title = "<incorrect instructions>";
+            m_mainFunction.clear();
             return false;
         }
     }
@@ -214,14 +220,13 @@ bool CScript::CheckToken()
 bool CScript::Compile()
 {
     std::vector<std::string> functionList;
-    int             i;
     std::string     p;
 
     m_error = CBot::CBotNoErr;
     m_cursor1 = 0;
     m_cursor2 = 0;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
+    m_title.clear();
+    m_mainFunction.clear();
     m_bCompile = false;
 
     if ( IsEmpty() )  // program exist?
@@ -239,33 +244,17 @@ bool CScript::Compile()
     {
         if (functionList.empty())
         {
-            strcpy(m_title, "<extern missing>");
-            m_mainFunction[0] = 0;
+            m_title = "<extern missing>";
+            m_mainFunction.clear();
         }
         else
         {
-            p = functionList[0];
-            i = 0;
-            bool titleDone = false;
-            while ( true )
+            m_mainFunction = functionList[0];
+            m_title = m_mainFunction;
+            if (m_title.length() >= 20)
             {
-                if ( p[i] == 0 || p[i] == '(' )  break;
-                if ( i >= 20 && !titleDone )
-                {
-                    m_title[i+0] = '.';
-                    m_title[i+1] = '.';
-                    m_title[i+2] = '.';
-                    m_title[i+3] = 0;
-                    titleDone = true;
-                }
-                if(!titleDone)
-                    m_title[i] = p[i];
-                m_mainFunction[i] = p[i];
-                i ++;
+                m_title = m_title.substr(0, 20)+"...";
             }
-            if(!titleDone)
-                m_title[i] = 0;
-            m_mainFunction[i] = p[i];
         }
         m_bCompile = true;
         return true;
@@ -283,8 +272,8 @@ bool CScript::Compile()
         {
             m_cursor1 = m_cursor2 = 0;
         }
-        strcpy(m_title, "<error>");
-        m_mainFunction[0] = 0;
+        m_title = "<error>";
+        m_mainFunction.clear();
         return false;
     }
 }
@@ -292,9 +281,9 @@ bool CScript::Compile()
 
 // Returns the title of the script.
 
-void CScript::GetTitle(char* buffer)
+const std::string& CScript::GetTitle()
 {
-    strcpy(buffer, m_title);
+    return m_title;
 }
 
 
@@ -317,9 +306,9 @@ bool CScript::Run()
 {
     if (m_botProg == nullptr)  return false;
     if ( m_script == nullptr || m_len == 0 )  return false;
-    if ( m_mainFunction[0] == 0 ) return false;
+    if ( m_mainFunction.empty() ) return false;
 
-    if ( !m_botProg->Start(m_mainFunction) )  return false;
+    if ( !m_botProg->Start(m_mainFunction.c_str()) )  return false;
 
     m_bRun = true;
     m_bContinue = false;
@@ -605,13 +594,13 @@ void CScript::UpdateList(Ui::CList* list)
 
 void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
 {
-    if (rangeEnd > edit->GetMaxChar())
-        rangeEnd = edit->GetMaxChar();
+    if (rangeEnd > edit->GetTextLength())
+        rangeEnd = edit->GetTextLength();
 
     edit->SetFormat(rangeStart, rangeEnd, Gfx::FONT_HIGHLIGHT_COMMENT); // anything not processed is a comment
 
     // NOTE: Images are registered as index in some array, and that can be 0 which normally ends the string!
-    std::string text = std::string(edit->GetText(), edit->GetMaxChar());
+    std::string text = edit->GetText();
     text = text.substr(rangeStart, rangeEnd-rangeStart);
 
     auto tokens = CBot::CBotToken::CompileTokens(text.c_str());
@@ -788,13 +777,28 @@ void CScript::GetError(std::string& error)
     }
     else
     {
-        if ( m_error == static_cast<CBot::CBotError>(ERR_OBLIGATORYTOKEN) )
+        if (m_error == static_cast<CBot::CBotError>(ERR_OBLIGATORYTOKEN))
         {
-            std::string s;
-            GetResource(RES_ERR, m_error, s);
-            error = StrUtils::Format(s.c_str(), m_token);
+            error = StrUtils::Format(ngettext(
+                "You have to use \"%1$s\" at least once in this exercise (used: %2$d)",
+                "You have to use \"%1$s\" at least %3$d times in this exercise (used: %2$d)",
+                m_tokenAllowed), m_token.c_str(), m_tokenUsed, m_tokenAllowed);
         }
-        else if ( m_error < 1000 )
+        else if (m_error == static_cast<CBot::CBotError>(ERR_PROHIBITEDTOKEN))
+        {
+            if (m_tokenAllowed == 0)
+            {
+                error = StrUtils::Format(gettext("You cannot use \"%s\" in this exercise (used: %d)"), m_token.c_str(), m_tokenUsed);
+            }
+            else
+            {
+                error = StrUtils::Format(ngettext(
+                    "You have to use \"%1$s\" at most once in this exercise (used: %2$d)",
+                    "You have to use \"%1$s\" at most %3$d times in this exercise (used: %2$d)",
+                    m_tokenAllowed), m_token.c_str(), m_tokenUsed, m_tokenAllowed);
+            }
+        }
+        else if (m_error < 1000)
         {
             GetResource(RES_ERR, m_error, error);
         }
@@ -925,7 +929,6 @@ bool CScript::SendScript(const char* text)
     if ( !Compile() )  return false;*/
 
     Ui::CEdit* edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
-    edit->SetMaxChar(Ui::EDITSTUDIOMAX);
     edit->SetAutoIndent(m_engine->GetEditIndentMode());
     edit->SetText(text, true);
     GetScript(edit);
@@ -945,7 +948,6 @@ bool CScript::ReadScript(const char* filename)
     m_script.reset();
 
     edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
-    edit->SetMaxChar(Ui::EDITSTUDIOMAX);
     edit->SetAutoIndent(m_engine->GetEditIndentMode());
     edit->ReadText(filename);
     GetScript(edit);
@@ -964,7 +966,6 @@ bool CScript::WriteScript(const char* filename)
     }
 
     Ui::CEdit* edit = m_interface->CreateEdit(Math::Point(0.0f, 0.0f), Math::Point(0.0f, 0.0f), 0, EVENT_EDIT9);
-    edit->SetMaxChar(Ui::EDITSTUDIOMAX);
     edit->SetAutoIndent(m_engine->GetEditIndentMode());
     edit->SetText(m_script.get());
     edit->WriteText(filename);
@@ -1018,12 +1019,12 @@ bool CScript::Compare(CScript* other)
 
 // Management of the file name when the script is saved.
 
-void CScript::SetFilename(char *filename)
+void CScript::SetFilename(const std::string& filename)
 {
-    strcpy(m_filename, filename);
+    m_filename = filename;
 }
 
-char* CScript::GetFilename()
+const std::string& CScript::GetFilename()
 {
     return m_filename;
 }
