@@ -25,7 +25,6 @@
 
 #include "common/event.h"
 #include "common/logger.h"
-#include "common/misc.h"
 #include "common/settings.h"
 
 #include "common/resources/resourcemanager.h"
@@ -63,6 +62,7 @@
 #include "ui/controls/window.h"
 
 #include <stdio.h>
+#include <ctime>
 
 
 namespace Ui
@@ -425,15 +425,9 @@ bool CStudio::EventFrame(const Event &event)
 
 // Indicates whether a character is part of a word.
 
-bool IsToken(int character)
+bool IsToken(char c)
 {
-    char    c;
-
-    c = tolower(GetNoAccent(character));
-
-    return ( (c >= 'a' && c <= 'z') ||
-             (c >= '0' && c <= '9') ||
-             c == '_' );
+    return ( isalnum(c) || c == '_' );
 }
 
 // Seeks if the cursor is on a keyword.
@@ -442,8 +436,8 @@ void CStudio::SearchToken(CEdit* edit)
 {
     ObjectType  type;
     int         len, cursor1, cursor2, i, character, level;
-    char*       text;
-    char        token[100];
+    std::string text;
+    std::string token( 100, '\0');
 
     text = edit->GetText();
     len  = edit->GetTextLength();
@@ -509,7 +503,7 @@ void CStudio::SearchToken(CEdit* edit)
     }
     token[i] = 0;
 
-    m_helpFilename = GetHelpFilename(token);
+    m_helpFilename = GetHelpFilename(token.c_str());
     if ( m_helpFilename.length() == 0 )
     {
         for ( i=0 ; i<OBJECT_MAX ; i++ )
@@ -518,30 +512,30 @@ void CStudio::SearchToken(CEdit* edit)
             text = const_cast<char *>(GetObjectName(type));
             if ( text[0] != 0 )
             {
-                if ( strcmp(token, text) == 0 )
+                if ( token == text )
                 {
                     m_helpFilename = GetHelpFilename(type);
-                    SetInfoText(std::string(token), true);
+                    SetInfoText(token, true);
                     return;
                 }
             }
             text = const_cast<char *>(GetObjectAlias(type));
             if ( text[0] != 0 )
             {
-                if ( strcmp(token, text) == 0 )
+                if ( token == text )
                 {
                     m_helpFilename = GetHelpFilename(type);
-                    SetInfoText(std::string(token), true);
+                    SetInfoText(token, true);
                     return;
                 }
             }
         }
     }
 
-    text = const_cast<char *>(GetHelpText(token));
+    text = const_cast<char *>(GetHelpText(token.c_str()));
     if ( text[0] == 0 && m_helpFilename.length() > 0 )
     {
-        SetInfoText(std::string(token), true);
+        SetInfoText(token, true);
     }
     else
     {
@@ -612,7 +606,6 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
     edit->SetState(STATE_SHADOW);
     edit->SetInsideScroll(false);
 //? if ( m_bRunning )  edit->SetEdit(false);
-    edit->SetMaxChar(EDITSTUDIOMAX);
     edit->SetFontType(Gfx::FONT_COURIER);
     edit->SetFontStretch(1.0f);
     edit->SetDisplaySpec(true);
@@ -882,7 +875,7 @@ void CStudio::AdjustEditScript()
 
 // Ends edition of a program.
 
-bool CStudio::StopEditScript(bool bCancel)
+bool CStudio::StopEditScript(bool closeWithErrors)
 {
     CWindow*    pw;
     CEdit*      edit;
@@ -890,7 +883,7 @@ bool CStudio::StopEditScript(bool bCancel)
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW3));
     if ( pw == nullptr )  return false;
 
-    if ( !bCancel && !m_script->IsRunning() )
+    if ( !closeWithErrors && !m_script->IsRunning() )
     {
         edit = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
         if ( edit != nullptr )
@@ -1034,17 +1027,17 @@ void CStudio::UpdateButtons()
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_CLONE));
     if ( button == nullptr )  return;
-    button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable && !m_bRunning));
 
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_COMPILE));
     if ( button == nullptr )  return;
-    button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable && !m_bRunning));
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_RUN));
     if ( button == nullptr )  return;
     button->SetIcon(m_bRunning?8:21);  // stop/run
-    button->SetState(STATE_ENABLE, m_program->runnable || m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable || m_bRunning));
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_REALTIME));
     if ( button == nullptr )  return;
@@ -1251,7 +1244,7 @@ void CStudio::AdjustDialog()
     CEdit*      pe;
     Math::Point     wpos, wdim, ppos, ddim;
     int         nli, nch;
-    char        name[100];
+    std::string        name;
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return;
@@ -1282,9 +1275,9 @@ void CStudio::AdjustDialog()
         {
             pli->SetPos(ppos);
             pli->SetDim(ddim);
-            pli->SetTabs(0, ddim.x-(50.0f+130.0f+16.0f)/640.0f);
+            pli->SetTabs(0, ddim.x-(50.0f+140.0f+16.0f)/640.0f);
             pli->SetTabs(1,  50.0f/640.0f, Gfx::TEXT_ALIGN_RIGHT);
-            pli->SetTabs(2, 130.0f/640.0f);
+            pli->SetTabs(2, 140.0f/640.0f);
 //?         pli->ShowSelect();
         }
 
@@ -1308,7 +1301,7 @@ void CStudio::AdjustDialog()
             pe->SetDim(ddim);
 
             nch = static_cast< int >((ddim.x*640.0f-22.0f)/8.0f);
-            pe->GetText(name, 100);
+            name = pe->GetText(100);
             pe->SetMaxChar(nch);
             name[nch] = 0;  // truncates the text according to max
             pe->SetText(name);
@@ -1473,7 +1466,7 @@ void CStudio::SetFilenameField(CEdit* edit, const std::string& filename)
             name = name.substr(0, edit->GetMaxChar());  // truncates according to max length
         }
     }
-    edit->SetText(name.c_str());
+    edit->SetText(name);
 }
 
 // Updates the list after a change in name.
@@ -1500,7 +1493,7 @@ void CStudio::UpdateDialogAction()
     CWindow*    pw;
     CEdit*      pe;
     CButton*    pb;
-    char        name[100];
+    std::string        name;
     int         len, i;
     bool        bError;
 
@@ -1511,8 +1504,8 @@ void CStudio::UpdateDialogAction()
     pb = static_cast< CButton* >(pw->SearchControl(EVENT_DIALOG_OK));
     if ( pb == nullptr )  return;
 
-    pe->GetText(name, 100);
-    len = strlen(name);
+    name = pe->GetText(100);
+    len = name.size();
     if ( len == 0 )
     {
         bError = true;
@@ -1564,7 +1557,7 @@ void CStudio::UpdateDialogPublic()
     if ( pl != nullptr )
     {
         // GetResource(RES_TEXT, RT_IO_LIST, name); // TODO: unused?
-        pl->SetName(SearchDirectory(false).c_str(), false);
+        pl->SetName(SearchDirectory(false), false);
     }
 }
 
@@ -1575,7 +1568,7 @@ void CStudio::UpdateDialogList()
     CWindow*        pw;
     CList*          pl;
     int             i = 0;
-    char            time[100];
+    char            timestr[100];
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return;
@@ -1590,8 +1583,9 @@ void CStudio::UpdateDialogList()
     for (auto& prog : programs)
     {
         std::ostringstream temp;
-        TimeToAscii(CResourceManager::GetLastModificationTime(SearchDirectory(false) + prog), time);
-        temp << prog << '\t' << CResourceManager::GetFileSize(SearchDirectory(false) + prog) << "  \t" << time;
+        time_t now = CResourceManager::GetLastModificationTime(SearchDirectory(false) + prog);
+        strftime(timestr, 99, "%x %X", localtime(&now));
+        temp << prog << '\t' << CResourceManager::GetFileSize(SearchDirectory(false) + prog) << "  \t" << timestr;
         pl->SetItemName(i++, temp.str().c_str());
     }
 }
@@ -1624,25 +1618,25 @@ bool CStudio::ReadProgram()
 {
     CWindow*    pw;
     CEdit*      pe;
-    char        filename[100];
-    char        dir[100];
-    char*       p;
+    std::string        filename;
+    std::string        dir;
+    size_t p;
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return false;
 
     pe = static_cast< CEdit* >(pw->SearchControl(EVENT_DIALOG_EDIT));
     if ( pe == nullptr )  return false;
-    pe->GetText(filename, 100);
-    if ( filename[0] == 0 )  return false;
+    filename = pe->GetText(100);
+    if ( filename.empty() )  return false;
 
-    p = strstr(filename, ".txt");
-    if ( p == nullptr || p != filename+strlen(filename)-4 )
+    p = filename.find(".txt");
+    if ( p == std::string::npos )
     {
-        strcat(filename, ".txt");
+        filename += ".txt";
     }
-    strcpy(dir, SearchDirectory(true).c_str());
-    strcat(dir, filename);
+    dir = SearchDirectory(true);
+    dir += filename;
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW3));
     if ( pw == nullptr )  return false;
@@ -1662,32 +1656,32 @@ bool CStudio::WriteProgram()
 {
     CWindow*    pw;
     CEdit*      pe;
-    char        filename[100];
-    char        dir[100];
-    char*       p;
+    std::string        filename;
+    std::string        dir;
+    size_t p;
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return false;
 
     pe = static_cast< CEdit* >(pw->SearchControl(EVENT_DIALOG_EDIT));
     if ( pe == nullptr )  return false;
-    pe->GetText(filename, 100);
-    if ( filename[0] == 0 )  return false;
+    filename = pe->GetText(100);
+    if ( filename.empty() )  return false;
 
-    p = strstr(filename, ".txt");
-    if ( p == nullptr || p != filename+strlen(filename)-4 )
+    p = filename.find(".txt");
+    if ( p == std::string::npos )
     {
-        strcat(filename, ".txt");
+        filename += ".txt";
     }
-    strcpy(dir, SearchDirectory(true).c_str());
-    strcat(dir, filename);
+    dir = SearchDirectory(true);
+    dir += filename;
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW3));
     if ( pw == nullptr )  return false;
     pe = static_cast< CEdit* >(pw->SearchControl(EVENT_STUDIO_EDIT));
     if ( pe == nullptr )  return false;
 
-    if ( !pe->WriteText(std::string(dir)) )  return false;
+    if ( !pe->WriteText(dir) )  return false;
 
     m_script->SetFilename(filename);
     return true;
